@@ -5,13 +5,13 @@ md_init <- function(algo){
 
 #' @useDynLib openssl R_md_feed
 md_feed <- function(md, data){
-  stopifnot(is(md, "md"))
+  stopifnot(inherits(md, "md"))
   stopifnot(is.raw(data))
   .Call(R_md_feed, md, data)
 }
 
 #' @useDynLib openssl R_md_final
 md_final <- function(md){
-  stopifnot(is(md, "md"))
+  stopifnot(inherits(md, "md"))
   .Call(R_md_final, md)
 }
